# - Find an LIBNOISE installation or build tree.

# When LIBNOISE is found, the LIBNOISEConfig.cmake file is sourced to setup the
# location and configuration of LIBNOISE.  Please read this file, or
# LIBNOISEConfig.cmake.in from the LIBNOISE source tree for the full list of
# definitions.  Of particular interest is LIBNOISE_USE_FILE, a CMake source file
# that can be included to set the include directories, library directories,
# and preprocessor macros.  In addition to the variables read from
# LIBNOISEConfig.cmake, this find module also defines
#
#  LIBNOISE_FOUND    - Whether LIBNOISE was found. If this is true, LIBNOISE_DIR is okay.
#
#  LIBNOISE_DIR      - The directory containing LIBNOISEConfig.cmake.  
#                  This is the only cache entry.
#  LIBNOISE_USE_FILE - The full path to the UseLIBNOISE.cmake file.
#

SET(LIBNOISE_DIR_STRING "directory containing LIBNOISEConfig.cmake.")

# Search only if the location is not already known.
IF(NOT LIBNOISE_DIR)
  # Get the system search path as a list.
  IF(UNIX)
    STRING(REGEX MATCHALL "[^:]+" LIBNOISE_DIR_SEARCH1 "$ENV{PATH}")
  ELSE(UNIX)
    STRING(REGEX REPLACE "\\\\" "/" LIBNOISE_DIR_SEARCH1 "$ENV{PATH}")
  ENDIF(UNIX)
  STRING(REGEX REPLACE "/;" ";" LIBNOISE_DIR_SEARCH2 ${LIBNOISE_DIR_SEARCH1})

  # Construct a set of paths relative to the system search path.
  SET(LIBNOISE_DIR_SEARCH "")
  FOREACH(dir ${LIBNOISE_DIR_SEARCH2})
    SET(LIBNOISE_DIR_SEARCH ${LIBNOISE_DIR_SEARCH} "${dir}/..")
  ENDFOREACH(dir)

  #
  # Look for an installation or build tree.
  #
  FIND_PATH(LIBNOISE_DIR LIBNOISEConfig.cmake
    # Look for an environment variable LIBNOISE_DIR.
    $ENV{LIBNOISE_DIR}

    # Look in places relative to the system executable search path.
    ${LIBNOISE_DIR_SEARCH}

    # Look in standard UNIX install locations.
    /usr/local/lib/libnoise
    /usr/lib/libnoise

    # Read from the CMakeSetup registry entries.  It is likely that
    # LIBNOISE will have been recently built.
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild1]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild2]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild3]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild4]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild5]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild6]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild7]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild8]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild9]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild10]

    # Help the user find it if we cannot.
    DOC "The ${LIBNOISE_DIR_STRING}"
  )
ENDIF(NOT LIBNOISE_DIR)

# If LIBNOISE was found, load the configuration file to get the rest of the settings.
IF(LIBNOISE_DIR)
  SET(LIBNOISE_FOUND "1")
  MESSAGE(FATAL_ERROR "LIBNOISE found...")
  INCLUDE("${LIBNOISE_DIR}/LIBNOISEConfig.cmake")
ELSE(LIBNOISE_DIR)
  SET(LIBNOISE_FOUND "0")
  IF(LIBNOISE_FIND_REQUIRED)
    MESSAGE(FATAL_ERROR "Please set LIBNOISE_DIR to the ${LIBNOISE_DIR_STRING}")
  ENDIF(LIBNOISE_FIND_REQUIRED)
ENDIF(LIBNOISE_DIR)
