# - Find an QWT installation or build tree.

# When QWT is found, the QWTConfig.cmake file is sourced to setup the
# location and configuration of QWT.  Please read this file, or
# QWTConfig.cmake.in from the QWT source tree for the full list of
# definitions.  Of particular interest is QWT_USE_FILE, a CMake source file
# that can be included to set the include directories, library directories,
# and preprocessor macros.  In addition to the variables read from
# QWTConfig.cmake, this find module also defines
#
#  QWT_FOUND    - Whether QWT was found. If this is true, QWT_DIR is okay.
#
#  QWT_DIR      - The directory containing QWTConfig.cmake.  
#                  This is the only cache entry.
#  QWT_USE_FILE - The full path to the UseQWT.cmake file.
#

SET(QWT_DIR_STRING "directory containing QWTConfig.cmake.")

# Search only if the location is not already known.
IF(NOT QWT_DIR)
  # Get the system search path as a list.
  IF(UNIX)
    STRING(REGEX MATCHALL "[^:]+" QWT_DIR_SEARCH1 "$ENV{PATH}")
  ELSE(UNIX)
    STRING(REGEX REPLACE "\\\\" "/" QWT_DIR_SEARCH1 "$ENV{PATH}")
  ENDIF(UNIX)
  STRING(REGEX REPLACE "/;" ";" QWT_DIR_SEARCH2 ${QWT_DIR_SEARCH1})

  # Construct a set of paths relative to the system search path.
  SET(QWT_DIR_SEARCH "")
  FOREACH(dir ${QWT_DIR_SEARCH2})
    SET(QWT_DIR_SEARCH ${QWT_DIR_SEARCH} "${dir}/..")
  ENDFOREACH(dir)

  #
  # Look for an installation or build tree.
  #
  FIND_PATH(QWT_DIR QWTConfig.cmake
    # Look for an environment variable QWT_DIR.
    $ENV{QWT_DIR}

    # Look in places relative to the system executable search path.
    ${QWT_DIR_SEARCH}

    # Look in standard UNIX install locations.
    /usr/local/lib/QWT
    /usr/lib/QWT

    # Read from the CMakeSetup registry entries.  It is likely that
    # QWT will have been recently built.
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild1]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild2]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild3]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild4]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild5]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild6]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild7]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild8]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild9]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild10]

    # Help the user find it if we cannot.
    DOC "The ${QWT_DIR_STRING}"
  )
ENDIF(NOT QWT_DIR)

# If QWT was found, load the configuration file to get the rest of the settings.
IF(QWT_DIR)
  SET(QWT_FOUND "1")
  MESSAGE(FATAL_ERROR "QWT found...")
  INCLUDE("${QWT_DIR}/QWTConfig.cmake")
ELSE(QWT_DIR)
  SET(QWT_FOUND "0")
  IF(QWT_FIND_REQUIRED)
    MESSAGE(FATAL_ERROR "Please set QWT_DIR to the ${QWT_DIR_STRING}")
  ENDIF(QWT_FIND_REQUIRED)
ENDIF(QWT_DIR)
