#ifndef KINECTOPENNIRIMAGESOURCE_H__
#define KINECTOPENNIRIMAGESOURCE_H__

// Include before ITK due to namespace issues
#include <XnCppWrapper.h>

#include "KinectImage.h"
#include "ritkRImageSource.h"

template<class TRImage, class TKinectImage>
class KinectOpenNIRImageSource : public ritk::RImageSource<TRImage>
{
public:
	/** @name Typedefs */
	//@{

	/** @name Standard typedefs */
	//@{
	typedef KinectOpenNIRImageSource			Self;
	typedef ritk::RImageSource<TRImage>			Superclass;
	typedef itk::SmartPointer<Self>				Pointer;
	typedef itk::SmartPointer<const Self>		ConstPointer;
	//@}

	/**	@name The RImage types */
	//@{
	typedef typename Superclass::RImageType		RImageType;
	typedef typename RImageType::Pointer		RImagePointer;
	typedef TKinectImage						KinectImageType;
	typedef typename KinectImageType::Pointer	KinectImagePointer;
	//@}

	//@}

	/// Object creation
	itkNewMacro(Self);

	/// RTTI
	itkTypeMacro(KinectOpenNIRImageSource, ritk::RImageSource);

	/** @brief	Generate the output information
	 *
	 *	@details
	 *	Prepare the allocation of the output image during the first back propagation of the pipeline.
	 *	In particular, the regions of the output image and the camera parameters are set.
	 */
	void GenerateOutputInformation();

	/**	@name Connect/disconnect */
	//@{
	void Connect();
	void Disconnect();
	//@}


protected:
	/// Constructor
	KinectOpenNIRImageSource();
	/// Destructorthe
	virtual ~KinectOpenNIRImageSource();

	/// Generate the range image here
	virtual void GenerateRImage();


private:
	/// purposely not implemented
	KinectOpenNIRImageSource(Self&); 
	/// purposely not implemented
	void operator=(const Self&); 

protected:
	/// The OpenNI context
	xn::Context m_Context;
	/// The depth generatro
	xn::DepthGenerator m_DepthGenerator;
	/// The image (RGB) generator
	xn::ImageGenerator m_ImageGenerator;
	/// The IR generator
	xn::IRGenerator m_IRGenerator;
	/// Connected flag
	bool m_Connected;
	/// Indicates that we're generating
	bool m_Generating;

	/** @name Real time clock */
	//@{
	typedef itk::RealTimeClock::TimeStampType TimeStampType;
	itk::RealTimeClock::Pointer m_Clock;
	//@}

};

/// Explicit template instantiation (default)
typedef KinectOpenNIRImageSource<ritk::RImageF2,KinectImageF2> KinectOpenNIRImageSourceF2;



#endif // KINECTOPENNIRIMAGESOURCE_H__
